// ==============================================================
//
// dm2_chutes.cpp
// Control module for dm2_chutes vessel class
//
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include <cstdio>
#include <cmath>
#include <string>
#include <stdio.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const double DROGUEMASS = 150;

static TOUCHDOWNVTX tdvtx_nogear[4] = {
	{_V( 0, 3, -2), 1e3, 1e2, 0.1, 0.1},
	{_V( 2.6, -1.5, -2), 1e3, 1e2, 0.1, 0.1},
	{_V(-2.6, -1.5, -2), 1e3, 1e2, 0.1, 0.1},
	{_V( 0, 0 , 20), 1e3, 1e2, 0.1}
};


void VLiftCoeff (double aoa, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};


	aoa += PI;
	int idx = max (0, min (15, (int)(aoa*istep)));
	double d = aoa*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	*cl = 0;
	
}

// 2. horizontal lift component (vertical stabiliser and body)

void HLiftCoeff (double beta, double M, double Re, double *cl, double *cm, double *cd)
{
	static const double step = RAD*22.5;
	static const double istep = 1.0/step;
	static const int nabsc = 17;
	static const double CL[nabsc] = {0, 0.1, 0.2, 0.1, 0, 0.1, 0.2, 0.1, 0, -0.1, -0.2, -0.1, 0, -0.1, -0.2, -0.1, 0};

	beta += PI;
	int idx = max (0, min (15, (int)(beta*istep)));
	double d = beta*istep - idx;
	*cl = CL[idx] + (CL[idx+1]-CL[idx])*d;
	*cm = 0.0;
	*cd = 1 + oapiGetInducedDrag (*cl, 0.16, 0.2);
	*cl = 0;
}



// ==============================================================
// Shuttle-PB class interface
// ==============================================================

class dm2_chutes: public VESSEL3 {
public:
	dm2_chutes (OBJHANDLE hVessel, int flightmodel);
	~dm2_chutes ();
	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	//////void clbkPostStep (double simt, double simdt, double mjd);
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	///int clbkConsumeDirectKey(char *kstate);
	//bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	void clbkPostCreation();
	
	
	
private:
	void DefineAnimations (void);
	MGROUP_TRANSFORM *collapse;
	UINT anim_chute_collapse;
	double collapse_proc;

};

dm2_chutes::dm2_chutes (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
    

}

dm2_chutes::~dm2_chutes ()
{
	delete collapse;
}

void dm2_chutes::DefineAnimations (void)
{
	 //collapse chute
	anim_chute_collapse = CreateAnimation (0);
	static UINT chtGrp1[2] = {0,1};
	collapse = new MGROUP_SCALE  (0, chtGrp1, 2, _V(0,0,-5), _V(2,2,0.0001));
	AddAnimationComponent (anim_chute_collapse, 0, 1, collapse);
}


// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void dm2_chutes::clbkSetClassCaps (FILEHANDLE cfg)
{
	collapse_proc = 0;

	// physical vessel parameters
	SetSize (10);
	SetEmptyMass (DROGUEMASS);
	SetPMI(_V(16,16,8));
	SetCrossSections (_V(3,3,3));
	SetCW(1,1,0.1,0.1);
	SetRotDrag (_V(1,1,1));
	SetTouchdownPoints ( tdvtx_nogear, 4);
	DefineAnimations ();

	// camera parameters
	SetCameraOffset (_V(0,0,0.1));

	SetMeshVisibilityMode(AddMesh("DM2/DM2_Chutes"),MESHVIS_ALWAYS);
	
	

	// ***************** thruster definitions *******************
	
	


	///////////////////////////////////////////////////////
	/////AirFoils

	/////  airfoils
	CreateAirfoil2 (LIFT_VERTICAL, _V(0,0,5), VLiftCoeff, 1, 15, 1);
	CreateAirfoil2 (LIFT_HORIZONTAL, _V(0,0,5), HLiftCoeff, 1, 15, 1);
	

	
}



// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new dm2_chutes (hvessel, flightmodel);
}

// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (dm2_chutes*)vessel;
}




void dm2_chutes::clbkSaveState (FILEHANDLE scn)
{
	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState(scn);

	sprintf (cbuf, "%0.4f", collapse_proc);
	oapiWriteScenario_string (scn, "ANMT", cbuf);

}


void dm2_chutes::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;
	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANMT", 4)) {
			sscanf (line+4, "%lf", &collapse_proc);
		} else {
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation(anim_chute_collapse, collapse_proc);
}



int dm2_chutes::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

	/////////////////////////////////



	//////////////////////////////////

  if (!down) return 0; // only process keydown events

 if (KEYMOD_SHIFT (kstate)) 
  {	 
	switch (key)
	{
		

	} 
  }


 else if (KEYMOD_CONTROL (kstate)) 
  { 
	switch (key)
	{
		
	} 
  }

else { // unmodified keys
    switch (key) 
		{
		//case OAPI_KEY_N: // nosecone
		//if (nc_status == 0) JettisonNC();
		//return 1;

		//case OAPI_KEY_M: // trunk
		//if (tk_status == 0) umb_status = 1;
		//return 1;

		}
  }
  return 0;
}



void dm2_chutes::clbkPostCreation()
{

	
}



void dm2_chutes::clbkPreStep(double SimT, double SimDT, double MJD)
{
	//if (GetAltitude(ALTMODE_GROUND) < 20) oapiDeleteVessel(GetHandle());
	double da = SimDT*0.2;
	collapse_proc = collapse_proc + da;
	if (collapse_proc > 1)collapse_proc = 1;
	SetAnimation(anim_chute_collapse, collapse_proc);
	if (collapse_proc == 1) oapiDeleteVessel(GetHandle());
}