// ==============================================================
//                 ORBITER MODULE: go_searcher
//                  
//
// go_searcher.cpp
// Control module for go_searcher vessel class
//
// 
// ==============================================================

#define STRICT
#define ORBITER_MODULE

#include "orbitersdk.h"
#include "OrbiterSoundSDK40.h"
#include <stdio.h>
#include <math.h>

// ==============================================================
// Some vessel parameters
// ==============================================================

const double RCS_ISP = 300000;
const double RCSTH = 3000;
const double MAINTH = 60000;
const double HOIST_SPEED = 0.05;
const double TROLLEY_SPEED = 0.1;
const double FUELMASS = 5000;
const double EMPTY_MASS = 20000;
//const double M_E = 2.71828182845904523536;

VECTOR3 pist_bottom = _V(0,1.354,-21.844);
VECTOR3 pist_top = _V(0,6.114,-21.84);//_V(0,6.422,-21.84);
VECTOR3 fd_att_dir = _V(0,1,0);

VECTOR3 bspos1 = _V(-2.118, 11.488, 16.378);
VECTOR3 bspos2 = _V( 2.118, 11.488, 16.378);
VECTOR3 bspos3 = _V( 3.21, 8.241, -13.307);
VECTOR3 bspos4 = _V(-3.21, 8.241, -13.307);
VECTOR3 bspos5 = _V( 0,12.297,10.119);
VECTOR3 bspos6 = _V(1.139,19.209,10.853);
VECTOR3 bscolr = _V(1,0,0);
VECTOR3 bscol2 = _V(1,1,1);
VECTOR3 bscolg = _V(0,1,0);

#define YELLOW RGB(255, 255, 80)
#define GREEN RGB(0, 225, 0)

static const DWORD ntdvtx = 3;
static TOUCHDOWNVTX tdvtx_geardown[ntdvtx] = {
	{_V(    0, -0.1, 10), 1e6, 1e5, 0.1, 0.1},
	{_V(-8.66, -0.1,-5), 1e6, 1e5, 0.1, 0.1},
	{_V( 8.66, -0.1,-5), 1e6, 1e5, 0.1, 0.1}
};


// Calculate lift coefficient [Cl] as a function of aoa (angle of attack) over -Pi ... Pi
// Implemented here as a piecewise linear function
void VLiftCoeff (VESSEL *v, double aoa, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 9;
	static const double AOA[nabsc] = {-180*RAD,-60*RAD,-30*RAD, -2*RAD, 10*RAD,15*RAD,50*RAD,90*RAD,180*RAD};
	static const double CL[nabsc]  = {0,0,0,0,0,0,0,0,0};
	static const double CM[nabsc]  = {0,0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && AOA[i+1] < aoa; i++);
	double f = (aoa-AOA[i]) / (AOA[i+1]-AOA[i]);
	*cl = 0;//CL[i] + (CL[i+1]-CL[i]) * f;  // aoa-dependent lift coefficient
	*cm = 0;//CM[i] + (CM[i+1]-CM[i]) * f;  // aoa-dependent moment coefficient
	double saoa = sin(aoa);
	double pd = 0.03 + 0.4*saoa*saoa;  // profile drag
	*cd = 0;// pd + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
	// profile drag + (lift-)induced drag + transonic/supersonic wave (compressibility) drag
}

void HLiftCoeff (VESSEL *v, double beta, double M, double Re, void *context, double *cl, double *cm, double *cd)
{
	int i;
	const int nabsc = 8;
	static const double BETA[nabsc] = {-180*RAD,-135*RAD,-90*RAD,-45*RAD,45*RAD,90*RAD,135*RAD,180*RAD};
	static const double CL[nabsc]   = {0,0,0,0,0,0,0,0};
	for (i = 0; i < nabsc-1 && BETA[i+1] < beta; i++);
	*cl = 0;//CL[i] + (CL[i+1]-CL[i]) * (beta-BETA[i]) / (BETA[i+1]-BETA[i]);
	*cm = 0.0;
	*cd = 0; //0.03 + oapiGetInducedDrag (*cl, 1.5, 0.7) + oapiGetWaveDrag (M, 0.75, 1.0, 1.1, 0.04);
}


class go_searcher: public VESSEL3 {
public:
	
	go_searcher (OBJHANDLE hVessel, int flightmodel);
	~go_searcher ();

	void clbkSetClassCaps (FILEHANDLE cfg);
	void clbkPreStep(double SimT, double SimDT, double MJD);
	void clbkPostStep (double simt, double simdt, double mjd);
	void clbkPostCreation();
	void clbkSaveState (FILEHANDLE scn);
	void clbkLoadStateEx (FILEHANDLE scn, void *status);
	int clbkConsumeBufferedKey(DWORD key, bool down, char *kstate);
	int clbkConsumeDirectKey(char *kstate);
	bool clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp);
	bool clbkLoadGenericCockpit ();
	int MyID;
	

private:
	void DefineAnimations (void);
	void Grapple(void);
	void GrappleTrolley(void);
	void SetCamDir(void);
	void CheckFD(void);
	void SpawnAS350(void);
	//void CalcFair1(void);
	//void CalcFair2(void);
	//void SpawnBalloon(void);
	//void TrackBalloon(void);

	THRUSTER_HANDLE th_rcs[6], th_group[2];
	PROPELLANT_HANDLE hProp;
	ATTACHMENTHANDLE hAtt_grapple, hAtt_trolley, hAtt_fd;
	ANIMATIONCOMPONENT_HANDLE Section1, Section2, Section3, Section4;
	MGROUP_TRANSFORM *Hoist[10], *Trolley, *TrolleyTip;
	UINT anim_hoist, anim_trolley, anim_pist_rot, anim_pist_tran;
	double hoist_proc, trolley_proc, pist_rot_proc, pist_tran_proc;
	int hoist_status, trolley_status, cam_status, light_status;
	BEACONLIGHTSPEC beac1, beac2, beac3, beac4, beac5, beac6;
	BEACONLIGHTSPEC fd_beac[4];
	VECTOR3 fd_beac_pos[4];
	SpotLight *spot_light;
	VECTOR3 hoist_tip[1], trolley_tip[1], pist_con[1];
	VECTOR3 fd_att_pos, fd_att_rot;
	int fd_status, spawn_sw;

};

go_searcher::go_searcher (OBJHANDLE hVessel, int flightmodel)
: VESSEL3 (hVessel, flightmodel)
{
}

go_searcher::~go_searcher ()
{
	for (int i = 0; i < 8; i++) 
	{
	delete Hoist[i];
	}
	delete Trolley;
	delete TrolleyTip;
}

// Define animation sequences for moving parts
// --------------------------------------------------------------
void go_searcher::DefineAnimations (void)
{
	// 1. Fair_Net
	anim_trolley = CreateAnimation (0);

	static UINT Grp1[2] = {5,6};
	Trolley = new MGROUP_TRANSLATE(0, Grp1, 2, _V(0,0,-12));
	AddAnimationComponent (anim_trolley, 0, 1, Trolley);

	TrolleyTip = new MGROUP_TRANSLATE(LOCALVERTEXLIST, MAKEGROUPARRAY(trolley_tip),1, _V(0,0,-12));
	AddAnimationComponent (anim_trolley, 0, 1, TrolleyTip);

	// 2.hoist
	anim_hoist = CreateAnimation (0.5);

	static UINT Grp2[2] = {17,23};
	Hoist[0] = new MGROUP_ROTATE(0, Grp2, 2, _V(0, 1.354, -24.505), _V(1,0,0), (float)( 75*RAD));
	Section1 = AddAnimationComponent (anim_hoist, 0, 1, Hoist[0]);

	static UINT Grp3[1] = {10};
	Hoist[1] = new MGROUP_ROTATE(0, Grp3, 1, _V(0, 12.328, -24.529), _V(1,0,0), (float)( -75*RAD));
	AddAnimationComponent (anim_hoist, 0, 1, Hoist[1], Section1);

	Hoist[2] = new MGROUP_ROTATE(LOCALVERTEXLIST, MAKEGROUPARRAY(hoist_tip), 1, _V(0, 12.328, -24.529), _V(1,0,0), (float)( -75*RAD));
	AddAnimationComponent (anim_hoist, 0, 1, Hoist[2], Section1);

	static UINT Grp4[2] = {21,22};
	Hoist[3] = new MGROUP_ROTATE(0, Grp4, 2, _V(0, 1.354, -19.506), _V(1,0,0), (float)( 75*RAD));
	Section2 = AddAnimationComponent (anim_hoist, 0, 1, Hoist[3]);

	Hoist[4] = new MGROUP_ROTATE(LOCALVERTEXLIST, MAKEGROUPARRAY(pist_con), 1, _V(0, 1.354, -19.506), _V(1,0,0), (float)( 75*RAD));
	AddAnimationComponent (anim_hoist, 0, 1, Hoist[4]);

	static UINT Grp5[2] = {15,16};
	Hoist[5] = new MGROUP_ROTATE(0, Grp5, 2, _V(0, 8.333, -19.506), _V(1,0,0), (float)( -82*RAD));
	AddAnimationComponent (anim_hoist, 0, 1, Hoist[5], Section2);

	/////// piston

	anim_pist_rot = CreateAnimation (0.5);

	static UINT Grp6[2] = {18,20};
	Hoist[6] = new MGROUP_ROTATE(0, Grp6, 2, _V(0, 1.354, -21.844), _V(1,0,0), (float)( -180*RAD));
	Section3 = AddAnimationComponent (anim_pist_rot, 0, 1, Hoist[6]);

	anim_pist_tran = CreateAnimation (0.0);

	static UINT Grp7[1] = {19};
	Hoist[7] = new MGROUP_TRANSLATE(0, Grp7, 1, _V(0, 10, 0));
	AddAnimationComponent (anim_pist_tran, 0, 1, Hoist[7], Section3);



	


}



// ==============================================================
// go_searcher class interface
// ==============================================================

// ==============================================================
// Overloaded callback functions
// ==============================================================

// --------------------------------------------------------------
// Set the capabilities of the vessel class
// --------------------------------------------------------------
void go_searcher::clbkSetClassCaps (FILEHANDLE cfg)
{
	hoist_status = 0;
	trolley_status = 0;
	cam_status = 0;
	light_status = 0;

	hoist_proc = 0.5;
	trolley_proc = 0;
	pist_rot_proc = 0.5;
	pist_tran_proc = 0;

	pist_con[0] = _V(0,7.064,-19.816);
	hoist_tip[0] = _V(0,5.5,-24.505);
	trolley_tip[0] = _V(0,5.3,-10.715);

	fd_att_pos = _V(0,8.488,-4.19);
	fd_att_rot = _V(0,0,1);
	fd_status = 0;
	spawn_sw = 0;

	fd_beac_pos[0] = _V(-8.5, 8.6, 0);
	fd_beac_pos[1] = _V( 8.5, 8.6, 0);
	fd_beac_pos[2] = _V(0, 8.6, -8.5);
	fd_beac_pos[3] = _V(0, 8.6, 8.5);
	

	MATRIX3 rotmat = rotm(_V(0,1,0), (45*RAD));
	for (int i = 0; i < 4; i++)
	{
		fd_beac_pos[i] = mul(rotmat,fd_beac_pos[i]);
		fd_beac_pos[i] = fd_beac_pos[i] + _V(0,0, -4.19);
	}

	
	DefineAnimations ();
	
	// physical specs
	SetCW (0.1, 0.1, 0.1, 0.1);
	SetRotDrag (_V(0.1,0.1,0.1));
	SetPMI (_V(20,20,20));
	SetCrossSections (_V(1,1,1));
	SetSize (30);
	SetEmptyMass (EMPTY_MASS);
	SetMeshVisibilityMode ((AddMesh ("GoSearcher/go_searcher")), MESHVIS_ALWAYS);
	//SetMeshVisibilityMode ((AddMesh ("Falcon9R_V12/go_searcher_cabin")), MESHVIS_COCKPIT);
	//SetMeshVisibilityMode ((AddMesh ("Falcon9R_V12/go_searcher_cabin_night")), MESHVIS_NEVER);
	//SetTouchdownPoints (_V(0, -0.0001, 8), _V(-4,-0.0001,-8), _V(4,-0.0001,-8));
	SetTouchdownPoints(tdvtx_geardown, ntdvtx);

	// propellant resources

	hProp = CreatePropellantResource (FUELMASS, FUELMASS);

	
	// ***************** thruster definitions *******************
	
	th_rcs[ 0] = CreateThruster (_V(0, 0,20), _V( 1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 1] = CreateThruster (_V(0, 0, 20), _V(-1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 2] = CreateThruster (_V(0, 0,-20), _V( 1, 0, 0), RCSTH, hProp, RCS_ISP);
	th_rcs[ 3] = CreateThruster (_V(0, 0,-20), _V(-1, 0, 0), RCSTH, hProp, RCS_ISP);

	th_rcs[ 4] = CreateThruster (_V(0,0, 0), _V( 0, 0, 1), MAINTH, hProp, RCS_ISP);
	th_rcs[ 5] = CreateThruster (_V(0,0, 0), _V( 0, 0,-1), MAINTH, hProp, RCS_ISP);
	

	th_group[0] = th_rcs[ 0];
	th_group[1] = th_rcs[ 3];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWRIGHT);

	th_group[0] = th_rcs[ 1];
	th_group[1] = th_rcs[ 2];
	CreateThrusterGroup (th_group, 2, THGROUP_ATT_YAWLEFT);

	


	th_group[0] = th_rcs[ 4];
	CreateThrusterGroup (th_group, 1, THGROUP_MAIN);

	th_group[0] = th_rcs[ 5];
	CreateThrusterGroup (th_group, 1, THGROUP_RETRO);



	///////////////////////////////////////////////////////
	/////AirFoils


	CreateAirfoil3 (LIFT_VERTICAL, _V(0,0,0), VLiftCoeff, 0, 0.1, 0.2, 0.1);
	CreateAirfoil3 (LIFT_HORIZONTAL, _V(0,0,0), HLiftCoeff, 0, 0.1, 0.2, 0.1);

	///////////////////////////////////////////////////////
	////// atts

	hAtt_trolley = CreateAttachment(false, trolley_tip[0], _V(0,-1, 0),_V(0, 0, -1), "XS");
	hAtt_grapple = CreateAttachment(false, hoist_tip[0], _V(0,-1, 0),_V(0, 0, -1), "XS");
	hAtt_fd = CreateAttachment(false, fd_att_pos, fd_att_dir,fd_att_rot, "XS");

	/////////////////////////
	SURFHANDLE smoke = oapiRegisterParticleTexture ("Contrail1");

	PARTICLESTREAMSPEC smoketrail = {
		0, 1.5, 6, 8, 0.08, 0.3, -0.2, 0, PARTICLESTREAMSPEC::EMISSIVE,
		PARTICLESTREAMSPEC::LVL_SQRT, 0, 1,
		PARTICLESTREAMSPEC::ATM_PLIN, 0.5, 1,
		smoke
	};

	AddExhaustStream(th_rcs[ 4], _V(-3,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(-2,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(-1,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(0,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(1,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(2,0.2,-26), &smoketrail);
	AddExhaustStream(th_rcs[ 4], _V(3,0.2,-26), &smoketrail);

	/*AddExhaustStream(th_rcs[ 5], _V(-3,0.2,19), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(-2,0.2,20), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(-1,0.2,21), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(0,0.2,22), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(1,0.2,21), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(2,0.2,20), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(3,0.2,19), &smoketrail);*/

	AddExhaustStream(th_rcs[ 5], _V(5.5,0.2,-24.5), &smoketrail);
	AddExhaustStream(th_rcs[ 5], _V(-5.5,0.2,-24.5), &smoketrail);

	///////////////////////////// LIGHTS

	beac1.shape = BEACONSHAPE_STAR;
	beac1.pos = &bspos1;
	beac1.col = &bscolr;
	beac1.size = 0.5;
	beac1.falloff = 0.5;
	beac1.period = 0;
	beac1.duration = 0;
	beac1.tofs = 0;
	beac1.active = false;
	AddBeacon (&beac1);

	beac2.shape = BEACONSHAPE_STAR;
	beac2.pos = &bspos2;
	beac2.col = &bscolg;
	beac2.size = 0.5;
	beac2.falloff = 0.5;
	beac2.period = 0;
	beac2.duration = 0;
	beac2.tofs = 0;
	beac2.active = false;
	AddBeacon (&beac2);

	beac3.shape = BEACONSHAPE_STAR;
	beac3.pos = &bspos3;
	beac3.col = &bscol2;
	beac3.size = 0.6;
	beac3.falloff = 0.5;
	beac3.period = 0;
	beac3.duration = 0;
	beac3.tofs = 0;
	beac3.active = false;
	AddBeacon (&beac3);

	beac4.shape = BEACONSHAPE_STAR;
	beac4.pos = &bspos4;
	beac4.col = &bscol2;
	beac4.size = 0.6;
	beac4.falloff = 0.5;
	beac4.period = 0;
	beac4.duration = 0;
	beac4.tofs = 0;
	beac4.active = false;
	AddBeacon (&beac4);

	beac5.shape = BEACONSHAPE_STAR;
	beac5.pos = &bspos5;
	beac5.col = &bscol2;
	beac5.size = 0.6;
	beac5.falloff = 0.5;
	beac5.period = 0;
	beac5.duration = 0;
	beac5.tofs = 0;
	beac5.active = false;
	AddBeacon (&beac5);

	beac6.shape = BEACONSHAPE_STAR;
	beac6.pos = &bspos6;
	beac6.col = &bscol2;
	beac6.size = 0.5;
	beac6.falloff = 0.5;
	beac6.period = 0;
	beac6.duration = 0;
	beac6.tofs = 0;
	beac6.active = false;
	AddBeacon (&beac6);

	for (int i = 0; i < 4; i++)
	{
	fd_beac[i].shape = BEACONSHAPE_STAR;
	fd_beac[i].pos = &fd_beac_pos[i];
	fd_beac[i].col = &bscol2;
	fd_beac[i].size = 0.4;
	fd_beac[i].falloff = 0.5;
	fd_beac[i].period = 2;
	fd_beac[i].duration = 0.1;
	fd_beac[i].tofs = 0;
	fd_beac[i].active = false;
	AddBeacon (&fd_beac[i]);
	}

	COLOUR4 col_d = {0.8,0.8,0.75,0};
	COLOUR4 col_s = {0.8,0.8,0.75,0};
	COLOUR4 col_a = {0,0,0,0};
	COLOUR4 col_white = {1,1,1,0};
	spot_light = (SpotLight*)AddSpotLight(_V(0,12.37,13.3), _V(0,0,-1), 500, 1.8e-4, 0.004, 1.8e-4, RAD*60, RAD*90, col_d, col_s, col_a);
	spot_light->Activate(false);
}

// ==============================================================
// API callback interface
// ==============================================================

// --------------------------------------------------------------
// Vessel initialisation
// --------------------------------------------------------------
DLLCLBK VESSEL *ovcInit (OBJHANDLE hvessel, int flightmodel)
{
	return new go_searcher (hvessel, flightmodel);
}




// --------------------------------------------------------------
// Vessel cleanup
// --------------------------------------------------------------
DLLCLBK void ovcExit (VESSEL *vessel)
{
	if (vessel) delete (go_searcher*)vessel;
}
// ==============================================================
// Overloaded clbkPostCreation callback functions
// ==============================================================

void go_searcher::clbkPostCreation ()
{
	MyID=ConnectToOrbiterSoundDLL(GetHandle());
	ReplaceStockSound(MyID,"Sound\\Vessel\\attsustain.wav",	REPLACE_MAIN_THRUST);
	SoundOptionOnOff(MyID,PLAYRADIOATC,FALSE);
	//SoundOptionOnOff3(MyID,PLAYWHENATTITUDEMODECHANGE,FALSE);
	SoundOptionOnOff(MyID,PLAYGPWS,FALSE);
	SoundOptionOnOff(MyID,PLAYCOUNTDOWNWHENTAKEOFF,FALSE);
	SoundOptionOnOff(MyID,PLAYLANDINGANDGROUNDSOUND,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINAIRCONDITIONING,FALSE);
	SoundOptionOnOff(MyID,PLAYCABINRANDOMAMBIANCE,FALSE);
	SoundOptionOnOff(MyID,DISPLAYTIMER,FALSE);
	SoundOptionOnOff(MyID,PLAYDOCKLANDCLEARANCE,FALSE);
	SoundOptionOnOff(MyID,PLAYRADARBIP,FALSE);
	
	//if (GetPropellantMass(hProp)< 0.001 && oapiGetSimTime() > 0.1)
	//{
	//	SetPropellantMass(hProp,FUELMASS);
	//}

	

}
void go_searcher::clbkSaveState (FILEHANDLE scn)
{

	VECTOR3 dir;
	GetAttachmentParams(hAtt_fd, fd_att_pos, dir, fd_att_rot);

	char cbuf[256];

	// default vessel parameters
	VESSEL3::clbkSaveState (scn);

	// custom parameters
	sprintf (cbuf, "%0.4f %d %0.4f", hoist_proc, trolley_status, trolley_proc);
	oapiWriteScenario_string (scn, "ANM", cbuf);

	sprintf (cbuf, "%0.4f %0.4f %d %d", pist_rot_proc, pist_tran_proc, cam_status, light_status);
	oapiWriteScenario_string (scn, "PST", cbuf);

	sprintf (cbuf, "%d %0.3f %0.3f %0.3f %0.4f %0.4f %0.4f", fd_status, fd_att_pos.x, fd_att_pos.y, fd_att_pos.z, fd_att_rot.x, fd_att_rot.y, fd_att_rot.z);
	oapiWriteScenario_string (scn, "FDK", cbuf);
	
}
void go_searcher::clbkLoadStateEx (FILEHANDLE scn, void *vs)
{
    char *line;

	while (oapiReadScenario_nextline (scn, line)) {
		if (!strnicmp (line, "ANM", 3))
		{
			sscanf (line+3, "%lf%d%lf", &hoist_proc, &trolley_status, &trolley_proc);
		}
		else if (!strnicmp (line, "PST", 3))
		{
			sscanf (line+3, "%lf%lf%d%d", &pist_rot_proc, &pist_tran_proc, &cam_status, &light_status);
		}
		else if (!strnicmp (line, "FDK", 3))
		{
			sscanf (line+3, "%d%lf%lf%lf%lf%lf%lf", &fd_status, &fd_att_pos.x, &fd_att_pos.y, &fd_att_pos.z, &fd_att_rot.x, &fd_att_rot.y, &fd_att_rot.z);
		}
		else
		{
            ParseScenarioLineEx (line, vs);
			// unrecognised option - pass to Orbiter's generic parser
        }
   }
	
	SetAnimation (anim_hoist, hoist_proc);
	SetAnimation (anim_trolley, trolley_proc);
	SetAnimation (anim_pist_rot, pist_rot_proc);
	SetAnimation (anim_pist_tran, pist_tran_proc);

	if (light_status == 1)
	{
		beac1.active = true;
		beac2.active = true;
		beac3.active = true;
		beac4.active = true;
		beac5.active = true;
		beac6.active = true;
		spot_light->Activate(true);
		//SetMeshVisibilityMode(1, MESHVIS_NEVER);
		//SetMeshVisibilityMode(2, MESHVIS_COCKPIT);
	}

	if(fd_status == 1)
	{
		for (int i = 0; i < 4; i++)
		{
			fd_beac[i].active = true;
		}
	}

	if (cam_status == 1)SetCamDir();

	SetAttachmentParams(hAtt_fd, fd_att_pos, fd_att_dir, fd_att_rot);
	
}






int go_searcher::clbkConsumeBufferedKey(DWORD key, bool down, char *kstate) {

  if (!down) return 0; // only process keydown events

  if (KEYMOD_SHIFT (kstate)) 
  {
	  switch (key) 
	  {
		
	  }
	  return true;
  } 
  else { // unmodified keys
    switch (key) {
	
	case OAPI_KEY_G: // grapple/release
		Grapple();
		return 1;

	case OAPI_KEY_J: // spawn as350
		spawn_sw = 1;
		return 1;

	case OAPI_KEY_K: // activate fd
		if(fd_status == 0)
		{
			fd_status = 1;
			for (int i = 0; i < 4; i++)
				{
				fd_beac[i].active = true;
				}
		}
		else 
		{
			fd_status = 0;
			for (int i = 0; i < 4; i++)
				{
				fd_beac[i].active = false;
				}
		}
		return 1;

	case OAPI_KEY_B: // trolley grapple/release
		GrappleTrolley();
		return 1;

	case OAPI_KEY_V: //  trolley
		if (trolley_status == 0)trolley_status = 1;
		else if (trolley_status == 1)trolley_status = 3;
		else if (trolley_status == 2)trolley_status = 3;
		else if (trolley_status == 3)trolley_status = 1;
		return 1;

	case OAPI_KEY_E: //  cam_status
		if (cam_status == 0)cam_status = 1;
		else if (cam_status == 1)cam_status = 0;
		SetCamDir();
		return 1;

	case OAPI_KEY_L: //  light_status
		RESETKEY (kstate, OAPI_KEY_L);
		if(light_status == 0)
		{
		light_status = 1;
		beac1.active = true;
		beac2.active = true;
		beac3.active = true;
		beac4.active = true;
		beac5.active = true;
		beac6.active = true;
		spot_light->Activate(true);
		//SetMeshVisibilityMode(1, MESHVIS_NEVER);
		//SetMeshVisibilityMode(2, MESHVIS_COCKPIT);
		}
		else
		{
		light_status = 0;
		beac1.active = false;
		beac2.active = false;
		beac3.active = false;
		beac4.active = false;
		beac5.active = false;
		beac6.active = false;
		spot_light->Activate(false);
		//SetMeshVisibilityMode(1, MESHVIS_COCKPIT);
		//SetMeshVisibilityMode(2, MESHVIS_NEVER);
		}
		return 1;
	
		return true;
	}
}
  
return 0;
}



int go_searcher::clbkConsumeDirectKey(char *kstate)
{
if (KEYMOD_CONTROL (kstate)) 
	{
	return 0;
	}
else if (KEYMOD_ALT (kstate)) 
	{
	return 0;
	}
else if (KEYMOD_SHIFT (kstate)) 
	{
	return 0;
	}


else
	{
	if (KEYDOWN (kstate, OAPI_KEY_N))
		{
		hoist_status = 1;
		return 1;
		}
	else if (KEYDOWN (kstate, OAPI_KEY_M))
		{
		hoist_status = -1;
		return 1;
		}
	}

return 0;
}


void go_searcher::clbkPreStep(double SimT, double SimDT, double MJD)
{
	//steptime = SimDT;


//sprintf(oapiDebugString(), "%d", ibsw);

	VECTOR3 surf_vel = _V(0,0,0);
	GetGroundspeedVector(FRAME_LOCAL,surf_vel);
	double speed = length(surf_vel);
	if(speed > 1)
	{
	if(speed > 20)speed = 20;
	VECTOR3 resist_force = -surf_vel;
	normalise(resist_force);
	resist_force.x = resist_force.x*speed*60000/16;
	resist_force.y = resist_force.y*speed*60000/16;
	resist_force.z = resist_force.z*speed*60000/16;
	AddForce(resist_force, _V(0,0,0));
	}



	/// animations
	if (hoist_status == 1)
	{
		double da = SimDT * HOIST_SPEED;
		hoist_proc = hoist_proc + da;
		if (hoist_proc > 1) hoist_proc = 1;
		hoist_status = 0;
	}
	else if (hoist_status == -1)
	{
		double da = SimDT * HOIST_SPEED;
		hoist_proc = hoist_proc - da;
		if (hoist_proc < 0) hoist_proc = 0;
		hoist_status = 0;
	}
	SetAnimation (anim_hoist, hoist_proc);



	if (trolley_status == 0) trolley_proc = 0;
	else if (trolley_status == 1)
	{
		double da = SimDT * TROLLEY_SPEED;
		trolley_proc = trolley_proc + da;
		if (trolley_proc > 1) 
		{
			trolley_proc = 1;
			trolley_status = 2;
		}
	}
	else if (trolley_status == 2)trolley_proc = 1;
	else if (trolley_status ==3)
	{
		double da = SimDT * TROLLEY_SPEED;
		trolley_proc = trolley_proc - da;
		if (trolley_proc < 0) 
		{
			trolley_proc = 0;
			trolley_status = 0;
		}
	}
	SetAnimation (anim_trolley, trolley_proc);

	//// set atts
	SetAttachmentParams(hAtt_grapple, hoist_tip[0],_V(0,-1,0), _V(0,0,-1));
	SetAttachmentParams(hAtt_trolley, trolley_tip[0],_V(0,-1,0), _V(0,0,-1));

	/////// piston
	VECTOR3 req_vec = pist_con[0] - pist_bottom;
	double angle = atan2(req_vec.y, req_vec.z);
	//sprintf(oapiDebugString(), "%0.4f", angle*DEG);

	if(angle < 0)angle = 0;
	else if(angle > PI)angle = PI;
	pist_rot_proc = angle/PI;

	double pist_length = length(pist_top - pist_bottom);
	double req_lngth = length(req_vec);
	double ext_length = req_lngth - pist_length;
	pist_tran_proc = ext_length/10;

	SetAnimation (anim_pist_rot, pist_rot_proc);
	SetAnimation (anim_pist_tran, pist_tran_proc);

	if(spawn_sw == 1) SpawnAS350();

	if(fd_status == 1)CheckFD();
	
}


void go_searcher::clbkPostStep (double simt, double simdt, double mjd)
{
	
}

bool go_searcher::clbkLoadGenericCockpit ()
{
	SetCamDir();
	return true;
}

void go_searcher::SetCamDir()
{
	VECTOR3 camdefdir = _V(0,0,1);
	VECTOR3 camdefpos = _V(0,9.5,14.5);
	if(cam_status == 1)
	{
		camdefdir = _V(0,0,-1);
		camdefpos = _V(0,9.5,9);
	}
	SetCameraDefaultDirection(camdefdir, 0);
	SetCameraOffset(camdefpos);
	oapiCameraAttach(GetHandle(), 2);
}


void go_searcher::Grapple ()
{

if(GetAttachmentStatus(hAtt_grapple))
	{
	DetachChild(hAtt_grapple);
	return;
	}

double grapple_dist_max = 100;

VECTOR3 att_pos_global = _V(0,0,0);
Local2Global(hoist_tip[0], att_pos_global);

int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
	{

	OBJHANDLE hVessel = oapiGetVesselByIndex(i);
	if (hVessel == GetHandle()) continue;

	VECTOR3 ves_pos_glob = _V(0,0,0);
	oapiGetGlobalPos(hVessel, &ves_pos_glob);
	if (dist(ves_pos_glob, att_pos_global) > grapple_dist_max)continue;

	OBJHANDLE hTgt = oapiGetVesselByIndex(i);
	VESSEL *v = oapiGetVesselInterface (hTgt);
	DWORD nAttach = v->AttachmentCount (true);

	for (DWORD j = 0; j < nAttach; j++)
		{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
		VECTOR3 pos, dir, rot;
		v->GetAttachmentParams(hAtt,pos,dir,rot);
		VECTOR3 tgt_att_pos_glob;
		v->Local2Global(pos,tgt_att_pos_glob);
		if(dist(tgt_att_pos_glob, att_pos_global) < 1.0)
			{
				AttachChild(hTgt, hAtt_grapple, hAtt);

			}
		}
	}
}

void go_searcher::GrappleTrolley ()
{

double grapple_dist_max = 20;

VECTOR3 att_pos_global = _V(0,0,0);
Local2Global(trolley_tip[0], att_pos_global);

int num_ves = oapiGetVesselCount();
for (int i = 0; i < num_ves; i++) 
	{

	OBJHANDLE hVessel = oapiGetVesselByIndex(i);
	if (hVessel == GetHandle()) continue;

	VECTOR3 ves_pos_glob = _V(0,0,0);
	oapiGetGlobalPos(hVessel, &ves_pos_glob);
	if (dist(ves_pos_glob, att_pos_global) > grapple_dist_max)continue;

	OBJHANDLE hTgt = oapiGetVesselByIndex(i);
	VESSEL *v = oapiGetVesselInterface (hTgt);
	DWORD nAttach = v->AttachmentCount (true);

	for (DWORD j = 0; j < nAttach; j++)
		{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);
		VECTOR3 pos, dir, rot;
		v->GetAttachmentParams(hAtt,pos,dir,rot);
		VECTOR3 tgt_att_pos_glob;
		v->Local2Global(pos,tgt_att_pos_glob);
		if(dist(tgt_att_pos_glob, att_pos_global) < 1.0)
			{
				AttachChild(hTgt, hAtt_trolley, hAtt);

			}
		}
	}
}


void go_searcher::CheckFD()
{
OBJHANDLE hChild = GetAttachmentStatus(hAtt_fd);
if(hChild != NULL)
	{
	VESSEL *v = oapiGetVesselInterface (hChild);
	VECTOR3 childforce;
	v->GetThrustVector(childforce);
	VECTOR3 childforce_hrz;
	v->HorizonRot(childforce, childforce_hrz);
	double weight = v->GetMass();
	weight = weight*9.82;
	//sprintf(oapiDebugString(), "%0.2f %0.2f %0.2f %0.2f", childforce_hrz.x, childforce_hrz.y, childforce_hrz.z, weight);
	if(childforce_hrz.y > weight) DetachChild(hAtt_fd); //fd_status = 2;// DetachChild(hChild);
	}
else
	{
	double ves_dist_max = 100;
	VECTOR3 fd_pos_global = _V(0,0,0);
	Local2Global(_V(0,8.488,-4.19), fd_pos_global);

	int num_ves = oapiGetVesselCount();
	for (int i = 0; i < num_ves; i++) 
		{
		OBJHANDLE hVessel = oapiGetVesselByIndex(i);
		if (hVessel == GetHandle()) continue; // not me

		VECTOR3 ves_pos_glob = _V(0,0,0);
		oapiGetGlobalPos(hVessel, &ves_pos_glob);
		if (dist(ves_pos_glob, fd_pos_global) > ves_dist_max)continue; // too far

		VECTOR3 rel_vel_vec;
		oapiGetRelativeVel(hVessel,GetHandle(), &rel_vel_vec);
		if(length(rel_vel_vec) > 5)continue; // too fast

		VESSEL *v = oapiGetVesselInterface (hVessel);

		VECTOR3 tgt_aspd_hrz;
		v->GetAirspeedVector(FRAME_HORIZON, tgt_aspd_hrz);
		if(tgt_aspd_hrz.y > 0) continue; // going up

		DWORD nAttach = v->AttachmentCount (true);
		for (DWORD j = 0; j < nAttach; j++)
			{
		ATTACHMENTHANDLE hAtt = v->GetAttachmentHandle (true, j);

		const char *id = v->GetAttachmentId (hAtt);
		if (strncmp (id, "FD", 2)) continue; // attachment point not compatible

		VECTOR3 pos, dir, rot;
		v->GetAttachmentParams(hAtt,pos,dir,rot);
		VECTOR3 tgt_att_pos_glob;
		v->Local2Global(pos,tgt_att_pos_glob);
		VECTOR3 tgt_att_pos_loc;
		Global2Local(tgt_att_pos_glob, tgt_att_pos_loc);
		if(abs(tgt_att_pos_loc.x) > 8) continue;
		else if(abs(tgt_att_pos_loc.y - 8.488) > 0.5) continue;
		else if(abs(tgt_att_pos_loc.z + 4.19) > 8) continue;
		else
				{
				VECTOR3 tgt_att_pos_flat = _V(tgt_att_pos_loc.x, 8.488, tgt_att_pos_loc.z);

				VECTOR3 rot_hrz, rot_loc;
				v->HorizonRot(rot, rot_hrz);
				HorizonInvRot(rot_hrz, rot_loc);
				VECTOR3 rot_loc_flat = _V(rot_loc.x, 0, rot_loc.z);
				normalise(rot_loc_flat);
				SetAttachmentParams(hAtt_fd, tgt_att_pos_flat, _V(0,1,0), rot_loc_flat);
				AttachChild(hVessel, hAtt_fd, hAtt);
				}
			}
		}
	}

}



void go_searcher::SpawnAS350()
{
	OBJHANDLE hChild = GetAttachmentStatus(hAtt_fd);
	if(hChild != NULL) 
	{
		spawn_sw = 0;
		return;
	}
	else
	{
	SetAttachmentParams(hAtt_fd, _V(0,8.488,-4.19), _V(0,1,0), _V(0,0,1));
	VESSELSTATUS vs;
	GetStatus(vs);
	Local2Rel (_V(0,8.488,-4.19), vs.rpos);
	hChild = oapiCreateVessel ("AS350", "AS350/as350", vs);
	VESSEL2* v = (VESSEL2*)oapiGetVesselInterface(hChild);
	ATTACHMENTHANDLE hAtt_child = v->GetAttachmentHandle(true,0);
	AttachChild(hChild, hAtt_fd, hAtt_child);
	spawn_sw = 0;
	return;
	}
}


bool go_searcher::clbkDrawHUD (int mode, const HUDPAINTSPEC *hps, oapi::Sketchpad *skp)
{
	// draw the default HUD
	VESSEL3::clbkDrawHUD (mode, hps, skp);

	int s = hps->H;
	double d = (s*0.00130208);
	int sw = ((hps->W));
	int lw =  (int)(16*sw/1024);
	int lwoffset = sw - (18*lw);
	int hlw = (int) (lw/2);

	//int chw = 16;

	
	char test[256];
	strcpy(test,"abcdefghijklmnopqrstuvwxyz");
	int it = skp->GetTextWidth(test, 26);
	it = (int)(it/26);
	int chw = it;

	int roxl = 0;
	int royl = 0;

	double ds = s;
	double dsw = sw;
	double sc_ratio = ds/dsw;


	if (sc_ratio < 0.7284)
	{
	roxl = (lw*10);
	royl = (int) (-88*d);
	}



	int wd = (int)(152*d);
	int wc = (int)(168*d);
	int w0 = (int)(184*d);
	int w1 = (int) (200*d);
	int w2 = (int)(216*d);
	int w3 = (int)(232*d);
	int w4 = (int)(248*d);
	int w5 = (int)(264*d);
	int w6 = (int)(280*d);
	int w7 = (int)(296*d);
	int w8 = (int)(312*d);
	int w9 = (int)(328*d);
	//int w10 = (int)(344*d);
	//int w11 = (int)(360*d);
	//int w12 = (int)(376*d);
	//int w13 = (int)(392*d);
	//int w14 = (int)(408*d);
	//int w15 = (int)(424*d);

	
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"N/M = Rotate Hoist +/-");
	skp->Text ((10+roxl), (wd+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"G = Hoist Grapple/Release");
	skp->Text ((10+roxl), (wc+royl), abuf,strlen(abuf));
	}
	
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"V = Deploy/Stow Capsule Carrier");
	skp->Text ((10+roxl), (w0+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"B = Capsule Carrier Grapple");
	skp->Text ((10+roxl), (w1+royl), abuf,strlen(abuf));
	}
	
	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"E = Set View Forward/Back");
	skp->Text ((10+roxl), (w2+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"L = Lights On/Off");
	skp->Text ((10+roxl), (w3+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	if(fd_status == 0)sprintf(abuf,"K = Activate Flight Deck: OFF");
	else sprintf(abuf,"K = Activate Flight Deck: ACTIVE");
	skp->Text ((10+roxl), (w4+royl), abuf,strlen(abuf));
	}

	{
	skp->SetTextColor(0x0066FF66);
	char abuf[256];
	sprintf(abuf,"J = Spawn AS350 Helicopter");
	skp->Text ((10+roxl), (w5+royl), abuf,strlen(abuf));
	}


	

	

	return true;
}

